pro wave2d_45x180
;
;  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;  *  PROGRAM WAVEG2D: PLOTS THE DEFORMED GRID OF A STRESS WAVE MOVING *
;  *  THRU A SQUARE QUADRILATERAL FINITE ELEMENT GRID WHERE A DEFORMED *
;  *  GRID IS DRAWN FROM NODAL DISPLACEMENTS WHICH ARE STORED IN A     *
;  *  FILE WHERE DISPLACEMENTS ARE GROUPED WITH A SPECIFIC TIME.       *
;  *     FORMAT(2I5,4F5.3) NUMBER OF NODES, NUMBER OF ELEMENTS,        *
;  *  SCALE FOR HORIZ. COORDINATES H(I), SCALE VOR VERT. COORDINATES   *
;  *  Z(I). COORDINATES OF THE ORIGIN OR THE FIRST NODE POINT WHICH    *
;  *  IS ASSUMED TO BE LOCATED IN THE LOWER LEFT-HAND-CORNER OF THE    *
;  *  GRID. WRITTEN BY R.D.KRIZ, SPRING 1988. BOULDER,COLORADO.        *
;  *********************************************************************
;  * Modified R.D.Kriz sprng 2000 VaTech: run on npib-apache web-server*
;  *********************************************************************
;  * Modified by R.D Kriz spring 2008 VaTech: use color to highlight   *
;  * simulated transducer boundary displacements and its' centerline   *
;  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;
openr,5,'pltfl.dat'
hi=fltarr(8326)
zi=fltarr(8326)
n1=intarr(8100)
n2=intarr(8100)
n3=intarr(8100)
n4=intarr(8100)
itrans=intarr(30)
file=string('')
f=string('')
dh=fltarr(8326)
dz=fltarr(8326)
maxd=fltarr(500)
;
;   READ HEADING FROM INPUT FILE
;
readf,5,f
;
;       READ SECOND LINE OF INPUT FOR GRID INFO.
;
;      NODES , NUMBER OF NODES
;      NELEM , NUMBER OF ELEMENTS
;      SCAH  , SCALE OF HORIZ. COORDINATES
;      SCAZ  , SCALE OF VERTL. COORDINATES
;      HOR   , HORIZ. COORD. OF 1RST NODE
;      ZOR   , VERT. COORD. OF 1RST NODE
;
readf,5,format='(3I6,4F8.0)',nodes,nelem,ndisp,scah,scaz,hor,zor
nodes=fix(nodes)
nelem=fix(nelem)
nodesm1=nodes-1
nelemm1=nelem-1
;
;  Define mins and maxs for X and Y coordinates
;
 min_y=-2.0 & max_y=50.0
 min_x=-2.0 & max_x=185.0
;
; Set X and Y data Scales as PV-Wave system variables
!X.S = [-min_x, 1.]/(max_x-min_x)
!Y.S = [-min_y, 1.]/(max_y-min_y)
;
;        READ NODE POINT COORDINATES
;
for ind=0,nodesm1 do begin
   readf,5,format='(12x,2F12.5)',z,h
   hi(ind)=h
   zi(ind)=z
endfor
   elmntsize=zi(1)-zi(0)
   print,'Element size=',elmntsize
;
;      READ ELEMENT AND ASSOCIATED NODE NO. TOPOLOGY
;
for iel=0,nelemm1 do begin
   readf,5,format='(6x,4I6)',a,b,c,d
   n1(iel)=a
   n2(iel)=b
   n3(iel)=c
   n4(iel)=d
endfor
;
;   READ THE NUMBER OF TIME STEPS AND MULT. FACTOR FOR NODAL DISPL.S
;
readf,5,format='(I3,E10.3)',nt,smult
;
;        Read transducer node numbers
;
for ind=1,ndisp do begin
   readf,5,format='(2i6)',tnd,iconstr
   itrans(ind)=tnd
endfor
;
;  For debugging purposes printout the grid nodes 
;  associated with the simulated transducer
;
   icntr=(ndisp-1)/2+1
   cnb=itrans(icntr)-1
   cnt=cnb+44
 print,'** for transducers simulated with odd number of nodes'
 print,'** total no. of transducer nodes=',ndisp
 print,'** i-th transducer center node=',icntr
 print,'** red line nodes: start node no.=',cnb,' end node no.=',cnt
;
;   Setup Z buffer for making animations 
;
 set_plot,'pm'
 TEK_COLOR
 openzbuff,1100,500
;
; load ascii-color-table
;
    l_color_nw,256,'ct.ascii'
;
;   LOOP THRU DISPL.S FOR EACH TIME STEP
;
    maxd(2)=1.0
for it=1,nt do begin
;
;       READ TIME FOR INCREMENT
;
   readf,5,format='(6x,e11.4)',time
;
; READ and SCALE NODAL DISPLACEMENTS
;
   for in=0,nodesm1 do begin
      readf,5,format='(7x,2(1x,e11.4))',a,b
      dh(in)=b
      dz(in)=a
   endfor
;
   maxdh=max(dh)
   maxdz=max(dz)
   maxd(it)=sqrt(maxdz^2+maxdh^2)
;
; SIT: (S)cale computed at (I)nitial (T)ime step
;  scale=5.0*elmntsize/maxd(2)
; SET: (S)cale computed at (E)ach (T)ime step
   scale=2.0*elmntsize/maxd(it)
;
;  print,'       it      time        maxdh        maxdz       maxd(it)     scale'
;  print,it,time,maxdh,maxdz,maxd(it),scale
;
   for in=0,nodesm1 do begin
       dh(in)=dh(in)*scale
       dz(in)=dz(in)*scale
   endfor
;
;  OPEN WINDOW FOR EACH TIME STEP
;
;  window,1,xsize=650,ysize=400,xpos=100,ypos=100
   erase,255
;
;  CALCULATE DISPLACED GLOBAL COORD.S FROM NODAL DISPLACEMENTS
;                    AND THEN PLOT ELEMENTS 
;
   for ie=0,nelemm1 do begin
      plots, [hi(n1(ie)-1)+dh(n1(ie)-1),hi(n2(ie)-1)+dh(n2(ie)-1)],$
             [zi(n1(ie)-1)+dz(n1(ie)-1),zi(n2(ie)-1)+dz(n2(ie)-1)],$
             color=253
      plots, [hi(n2(ie)-1)+dh(n2(ie)-1),hi(n3(ie)-1)+dh(n3(ie)-1)],$
             [zi(n2(ie)-1)+dz(n2(ie)-1),zi(n3(ie)-1)+dz(n3(ie)-1)],$
             color=253
      plots, [hi(n3(ie)-1)+dh(n3(ie)-1),hi(n4(ie)-1)+dh(n4(ie)-1)],$
             [zi(n3(ie)-1)+dz(n3(ie)-1),zi(n4(ie)-1)+dz(n4(ie)-1)],$
             color=253
      plots, [hi(n4(ie)-1)+dh(n4(ie)-1),hi(n1(ie)-1)+dh(n1(ie)-1)],$
             [zi(n4(ie)-1)+dz(n4(ie)-1),zi(n1(ie)-1)+dz(n1(ie)-1)],$
             color=253
   endfor
;
;  Special case to highlight the location of prescribed boundary 
;  element displacements in color that reveal the deformation 
;  mechanics of the simulated boundary transducer.
;
;      purple (color=0), blue (color=25) or green (color=125)
;
   for ind=1,ndisp-1 do begin
     ind1m1=itrans(ind)-1
     ind2m1=itrans(ind+1)-1
     plots, [hi(ind1m1)+dh(ind1m1),hi(ind2m1)+dh(ind2m1)],$
            [zi(ind1m1)+dz(ind1m1),zi(ind2m1)+dz(ind2m1)],color=125
;
     ind1=itrans(ind)
     ind2=itrans(ind+1)
     plots, [hi(ind1)+dh(ind1),hi(ind2)+dh(ind2)],$
            [zi(ind1)+dz(ind1),zi(ind2)+dz(ind2)],color=0
;
     plots, [hi(ind1m1)+dh(ind1m1),hi(ind1)+dh(ind1)],$
            [zi(ind1m1)+dz(ind1m1),zi(ind1)+dz(ind1)],color=125
   endfor 
;
;  Plot vertical RED line thru center of simulated transducer.
;
   for ind=cnb,cnt do begin
      plots, [hi(ind)+dh(ind),hi(ind+1)+dh(ind+1)],$
             [zi(ind)+dz(ind),zi(ind+1)+dz(ind+1)],color=252
   endfor
;
;
;  WRITE TIME ON EACH PLOT
;
   XYOUTS,hi(nodesm1-1)-115.0,zi(nodesm1-1)+3.0,'  Time',SIZE=2,$
          color=253
   XYOUTS,hi(nodesm1-1)-103.0,zi(nodesm1-1)+3.0,' (seconds)',SIZE=1.5,$
          color=253
   XYOUTS,hi(nodesm1-1)-88.0,zi(nodesm1-1)+3.0,time,SIZE=1.2,$
          color=253
empty
;
;  Create series of GIF images for animation
;
 openw,4,'temp.dat'
 if (it lt 10) then printf,4,format='(i1)',it
 if ((it ge 10) and (it lt 100)) then printf,4,format='(i2)',it
 if (it ge 100) then printf,4,format='(i3)',it
 close,4
 openr,4,'temp.dat'
 nn=string('')
 readf,4,nn
 if (it lt 10) then file="a00"+nn+".gif"
 if ((it ge 10) and (it lt 100)) then file="a0"+nn+".gif"
 if (it ge 100) then file="a"+nn+".gif"
 close,4
 closezbuff,file
;
endfor 
;       END ALL PLOTTING
close,5
end 
